import sys
import time
from os.path import realpath, dirname, join, basename

import cv2  # imread
import torch
import numpy as np

import torch._utils
try:
    torch._utils._rebuild_tensor_v2
except AttributeError:
    def _rebuild_tensor_v2(storage, storage_offset, size, stride, requires_grad, backward_hooks):
        tensor = torch._utils._rebuild_tensor(storage, storage_offset, size, stride)
        tensor.requires_grad = requires_grad
        tensor._backward_hooks = backward_hooks
        return tensor
    torch._utils._rebuild_tensor_v2 = _rebuild_tensor_v2
import vot
from vot import Rectangle
from net import xSiamRPN
# from run_xSiamRPN import SiamRPN_init, SiamRPN_track
from run_xSiamRPN_bayesian import bayesian_SiamRPN_init as SiamRPN_init
from run_xSiamRPN_bayesian import bayesian_SiamRPN_track as SiamRPN_track
from utils import get_axis_aligned_bbox, cxy_wh_2_rect

class TrackerConfig(object):
    def __init__(self,window_influence=0.42,lr=0.25,penalty_k=0.04):
        self.exemplar_size = 127# input z size
        # self.instance_size = 271# input x size (search region)
        self.total_stride = 8
        self.sigma=5
        self.score_size = 25
        self.context_amount = 0.5  # context amount for the exemplar
        self.penalty_k = penalty_k # 0.04
        self.window_influence = window_influence
        self.lr = lr  # 0.25
# load net
net_file = join(realpath(dirname(__file__)), 'weight.pth')
net = xSiamRPN()
net.load_state_dict(torch.load(net_file)['state_dict'])
net.eval().cuda()

# warm up
for i in range(10):
    net.temple(torch.autograd.Variable(torch.FloatTensor(1, 3, 127, 127)).cuda())
    net(torch.autograd.Variable(torch.FloatTensor(1, 3, 255, 255)).cuda())

# start to track
handle = vot.VOT("polygon")
Polygon = handle.region()
cx, cy, w, h = get_axis_aligned_bbox(Polygon)

image_file = handle.frame()
if not image_file:
    sys.exit(0)
target_pos, target_sz = np.array([cx, cy]), np.array([w, h])
im = cv2.imread(image_file)  # HxWxC
state = SiamRPN_init(im, target_pos, target_sz, net,TrackerConfig())  # init tracker

while True:
    image_file = handle.frame()
    if not image_file:
        break
    im = cv2.imread(image_file)  # HxWxC
    state = SiamRPN_track(state, im)  # track
    res = cxy_wh_2_rect(state['target_pos'], state['target_sz'])
    handle.report(Rectangle(res[0], res[1], res[2], res[3]))

